<?php
/**                  
* Custom Schedule and automation functionality for this package. 
* Do not mistake for the class-automation.php file and class which 
* is global to all WTG plugins.
* 
* This class bridges CSV 2 POST and WEBTECHGLOBAL_Automation class so it can
* be edited where as WEBTECHGLOBAL_Automation class should never be edited.
* 
* Add a function for each automated action to this file. This is where
* the WEBTECHGLOBAL_Automation class can find functions and call them using
* values in data i.e. action name will match a method in this class.
* 
* @package WebTechGlobal WordPress Plugins
* @author Ryan Bayne   
* @version 1.0
*/

// load in WordPress only
defined( 'ABSPATH' ) || die( 'No direct script access allowed!' );

class CSV2POST_Schedule extends WEBTECHGLOBAL_Automation{
    public $schedule_array = array();
    
    public function __construct() {
        $this->CONFIG = new CSV2POST_Configuration();
        $this->DB = $this->CONFIG->load_class( 'CSV2POST_DB', 'class-wpdb.php', 'classes' );
        $this->DATA = $this->CONFIG->load_class( 'CSV2POST_Data', 'class-data.php', 'classes' );
        $this->LOG = $this->CONFIG->load_class( 'CSV2POST_Log', 'class-log.php', 'classes' );
        $this->UI = $this->CONFIG->load_class( 'CSV2POST_UI', 'class-ui.php', 'classes' );

        // attempt to get global schedule settings first
        $this->schedule_array = maybe_unserialize( get_option( 'webtechglobal_schedule' ) );

        // if no global schedule settings (introduced september 2015) then
        // get the original schedule array
        if( empty( $this->schedule_array ) ) {
            $this->schedule_array = maybe_unserialize( get_option( 'csv2post_schedule' ) );
        }
    }
               
    /**
    * Register this package within WEBTECHGLOBAL_Automation system and
    * allow the global schedule to be applied.
    * 
    * This does not mean CSV 2 POST can only do what WEBTECHGLOBAL_Automation
    * offers. The automation class from WTG has the purpose of balancing the
    * automation of multiple plugins/themes. It should not be edited and so
    * cannot provide functionality specific to CSV 2 POST. That is what
    * class-schedule.php is for.
    * 
    * @author Ryan R. Bayne
    * @package WebTechGlobal WordPress Plugins
    * @version 1.0
    * 
    * @todo change $plugin_weight to option/setting with a list of 
    * all weights in a single form so the user can order them properly.
    */    
    public function register_for_automation_part2() {
           
        // tell WEBTECHGLOBAL_Automation about the package
        $plugin_title = WTG_CSV2POST_TITLE;
        $plugin_name = WTG_CSV2POST_BASENAME;
        $plugin_abspath = WTG_CSV2POST_ABSPATH;
        $plugin_status = true;// false disables automation for this plugin
        $plugin_weight = 20;
        
        // some tidying 
        $package_atts = array(
            'plugin_title' => $plugin_title,    
            'plugin_name' => $plugin_name,    
            'plugin_abspath' => $plugin_abspath,    
            'plugin_status' => $plugin_status,    
            'plugin_weight' => $plugin_weight,    
        );
        
        // now WEBTECHGLOBAL_Automation actually comes into play
        // this will register the plugin only - no automation trigger
        // all plugins must register before automation called
        $this->register_for_automation_part3( $package_atts, $this->schedule_array['eventtypes'] );    
    }   

    /**
    * Import data from any waiting file. 
    * 
    * This is used within automated systems and so no part requires
    * a user and it takes into consideration that between each event
    * things may have changed about data or settings.
    * 
    * Remember create_notice() will not work if user ID cannot be applied.
    * So use of create_notice() in automation functions servers for testing
    * right now and doubles as a method to log.
    * 
    * @author Ryan R. Bayne
    * @package WebTechGlobal WordPress Plugins
    * @version 1.0
    */
    public function automatic_dataimport() {
        // get all datasources with new files that have not been 100% imported
        $waiting_files_array = $this->DB->find_waiting_files_new();
        if( !array( $waiting_files_array ) ) {
            // although an automated function, it can be called in testing
            // no notice is created during automation because noone is logged in and on admin
            $this->UI->create_notice( __( 'All .csv files have been fully imported
            or have moved onto an updating phase where data previously imported
            may be written over. Data updating is handled using other forms and proceures.', 'csv2post' ), 
            'info', 'Small', __( 'No Incomplete Datasources', 'csv2post' ) );
            return false;
        }
        
        // get any sources ID from those returned
        foreach( $waiting_files_array as $array_key => $data_source ) {
            if( !isset( $waiting_files_array[$array_key]['sourceid'] )
                || !isset( $waiting_files_array[$array_key]['sourceid'] ) ) {
                continue;
            }
            
            $source_id = $waiting_files_array[$array_key]['sourceid'];
            $project_id = $waiting_files_array[$array_key]['projectid']; 
            
            break;
        }
        
        if( !$waiting_files_array ) {
            // notice for testing and log
            $this->UI->create_notice( __( 'There are no new files awaiting
            their initial data import. If you have old files or files that may
            have changed since being registered. Please use Data Update tools This
            ensures correct procedures are used.', 'csv2post' ), 
            'info', 'Small', __( 'No New Files', 'csv2post' ) );            
            return false;
        }
        
        // log
        $atts = array(
            'outcome' => 1,
            'line' => __LINE__, 
            'function' => __FUNCTION__,
            'file' => __FILE__,           
            'type' => 'general',
            'category' => 'dataevent',
            
            'action' => __( "Automatic data import is using source $source_id 
            for project $project_id and will attempt data import.", 'csv2post' ),
            
            'priority' => 'normal',                        
            'triga' => 'automatic'
        );
        
        $this->CONFIG->newlog( $atts );
                
        // import data
        // TODO 3 c automation task: add control over the import limit
        $this->DATA->import_from_csv_file( $source_id, $project_id, 'import', 100 );        
    }

    /**
    * Update data using any new file.
    * 
    * @author Ryan R. Bayne
    * @package WebTechGlobal WordPress Plugins
    * @version 1.0
    */
    public function automatic_dataupdate() {
        // get datasources with old files that still need their update complete
        $waiting_files_array = $this->DB->find_waiting_files_old();
        if( !array( $waiting_files_array ) ) {
            $this->UI->create_notice( __( 'All datasources with changed .csv files
            have been fully re-imported.', 'csv2post' ), 'info', 'Small', 
            __( 'All Data Updated', 'csv2post' ) );
            return false;
        }
        
        if( !$waiting_files_array ) {
            // notice for testing and log
            $this->UI->create_notice( __( 'There are no old files waiting
            their data update. If you feel this is incorrect please seek
            help in the WebTechGlobal Forum.', 'csv2post' ), 
            'info', 'Small', __( 'No Changed Files', 'csv2post' ) );            
            return false;
        }
        
        // get any sources ID from those returned
        foreach( $waiting_files_array as $array_key => $data_source ) {
            
            if( !isset( $waiting_files_array[$array_key]['sourceid'] )
                || !isset( $waiting_files_array[$array_key]['sourceid'] ) ) {
                continue;
            }
            
            $source_id = $waiting_files_array[$array_key]['sourceid'];
            $project_id = $waiting_files_array[$array_key]['projectid']; 
            
            break;
        }
        
        // log
        $atts = array(
            'outcome' => 1,
            'line' => __LINE__, 
            'function' => __FUNCTION__,
            'file' => __FILE__,           
            'type' => 'general',
            'category' => 'dataevent',
            'action' => __( "Automatic data update is using source $source_id 
            for project $project_id and will attempt data re-import.", 'csv2post' ),        
            'priority' => 'normal',                        
            'triga' => 'automatic'
        );                                                                          
        
        $this->CONFIG->newlog( $atts );        
                
        // import data
        // TODO 3 c automation task: add control over the import limit
        $this->DATA->import_from_csv_file( $source_id, $project_id, 'update', 100 );
            
    }                                            

    /**
    * Create post using any available data for any project.
    * 
    * @author Ryan R. Bayne
    * @package WebTechGlobal WordPress Plugins
    * @version 1.0
    * 
    * @todo this process queries for sources records twice because it needs to
    * establish what source can be used, then get it's project and then work
    * from project forwards as per usual. So I have left get_unused_rows()
    * in use. Instead get_sources_with_updated_rows() could have a parameter 
    * to return the actual data. I also think by default it should return data
    * if only 1 source has been requested.
    */
    public function automatic_postcreation() {
        global $csv2post_settings;
        
        // get a source with unused rows, get it's project ID
        // this function gets active projects first
        $source = $this->DB->get_sources_with_unused_rows( 1 );
        
        if( !$source ) { 
            $this->UI->create_notice( __( 'You have used all imported rows from
            all datasources linked to active project. Import more data to create
            more posts. Note that this notice will not be created during actual
            automation and is test information only.' ), 'info', 'Small', 
            __( 'Testing Complete: No Rows Available', 'csv2post' ) );            
            return false; 
        }
        
        $autoblog = new CSV2POST_InsertPost();
        $autoblog->settings = $csv2post_settings;
        $autoblog->currentproject = $csv2post_settings['currentproject'];
        $autoblog->project = $this->DB->get_project( $source[0]['projectid'] );// gets project row, includes sources and settings
        $autoblog->projectid = $source[0]['projectid'];
        $autoblog->maintable = $this->DB->get_project_main_table( $source[0]['projectid'] );// main table holds post_id and progress statistics
        $autoblog->projectsettings = maybe_unserialize( $autoblog->project->projectsettings );// unserialize settings
        $autoblog->projectcolumns = $this->DB->get_project_columns_from_db( $source[0]['projectid'] );
        $autoblog->requestmethod = 'automatic';
        $autoblog->mainsourceid = $source[0]['sourceid'];// update the main source database table per post with new post ID
        unset( $autoblog->project->projectsettings);// just simplifying the project object by removing the project settings

        $idcolumn = false;
        if( isset( $autoblog->projectsettings['idcolumn'] ) ){
            $idcolumn = $autoblog->projectsettings['idcolumn'];    
        }
        
        // get rows not used to create posts
        // TODO 2 Task: apply users setting for automation event limit (replace 10)
        $unused_rows = $this->DB->get_unused_rows( $source[0]['projectid'], 10, $idcolumn );  
        if(!$unused_rows){
            // this should not happen within this automated function 
            $this->UI->create_notice( __( 'You have used all imported rows to 
            create posts. Please ensure you have imported all of your data if 
            you expected more posts than CSV 2 POST has already created.' ), 'info', 'Small', 
            __( 'No Rows Available', 'csv2post' ) );
            return;
        }
        
        // we will control how and when we end the operation
        // when true, output will be complete and foreach below will discontinue, 
        // this can be happen if maximum execution time is reached
        $autoblog->finished = false;
        
        $foreach_done = 0;
        foreach( $unused_rows as $key => $row ){
            ++$foreach_done;
                    
            // to get the output at the end, tell the class we are on the final post,
            // only required in "manual" requestmethod
            // TODO 2 Task: replace "10" with the users automation limit.
            if( $foreach_done == 10 ){    
                $autoblog->finished = true;// not completely finished, indicates this is the last post
            }
            
            // pass row to $autob
            $autoblog->row = $row; 
               
            // create a post
            $autoblog->start();
        }
        
        // log
        $atts = array(
            'outcome' => 1,
            'line' => __LINE__, 
            'function' => __FUNCTION__,
            'file' => __FILE__,           
            'type' => 'general',
            'category' => 'postsevent',
            'action' => __( "Up to $foreach_done posts may have been created.", 'csv2post' ),
            'priority' => 'normal',                        
            'triga' => 'manualrequest'
        );

        $this->LOG->newlog( $atts );
        
        $this->UI->create_notice( __( "Normally automated functions would not
        generate a user notice but this is to make testing easier. The post
        creation process is complete and up to $foreach_done posts may have
        been created." ), 'info', 'Small', 
        __( 'Testing Complete: Posts Created', 'csv2post' ) );        
    } 
    
    /**
    * Automation function for updating posts per schedule.
    * 
    * Includes notices for users who
    * are testing the function directly.
    * 
    * @author Ryan R. Bayne
    * @package WebTechGlobal WordPress Plugins
    * @version 1.0
    * 
    * @todo this process queries for sources records twice because it needs to
    * establish what source can be used, then get it's project and then work
    * from project forwards as per usual. So I have left get_updated_rows()
    * in use. Instead get_sources_with_updated_rows() could have a parameter 
    * to return the actual data. I also think by default it should return data
    * if only 1 source has been requested.  
    */
    public function automatic_postupdate() {
        global $csv2post_settings;

        // TODO 2 Task: apply users setting for automated event limit here (replace 5)    
        $total_posts = 5;
        
        $source = $this->DB->get_sources_with_updated_rows( 1 );

        if( !$source ) { 
            $this->UI->create_notice( __( 'All imported rows have been used to 
            create and update posts. No posts have been updated in this test.' ), 'info', 'Small', 
            __( 'Testing Complete: No Posts Updated', 'csv2post' ) );            
            return false; 
        }
                    
        $autoblog = new CSV2POST_UpdatePost();
        $autoblog->settings = $csv2post_settings;
        $autoblog->currentproject = $csv2post_settings['currentproject'];// dont automatically use by default, request may be for specific project
        $autoblog->project = $this->DB->get_project( $source[0]['projectid'] );// gets project row, includes sources and settings
        $autoblog->projectid = $source[0]['projectid'];
        $autoblog->maintable = $this->DB->get_project_main_table( $source[0]['projectid'] );// main table holds post_id and progress statistics
        $autoblog->projectsettings = maybe_unserialize( $autoblog->project->projectsettings);// unserialize settings
        $autoblog->projectcolumns = $this->DB->get_project_columns_from_db( $source[0]['projectid'] );
        $autoblog->requestmethod = 'manual';
        $sourceid_array = $this->DB->get_project_sourcesid( $source[0]['projectid'] );
        $autoblog->mainsourceid = $sourceid_array[0];// update the main source database table per post with new post ID
        unset( $autoblog->project->projectsettings );// simplifying the object by removing the project settings

        // we will control how and when we end the operation
        $autoblog->finished = false;// when true, output will be complete and foreach below will discontinue, this can be happen if maximum execution time is reached
        
        $idcolumn = false;
        if( isset( $autoblog->projectsettings['idcolumn'] ) ){
            $idcolumn = $autoblog->projectsettings['idcolumn'];    
        }
                               
        $updated_rows = $this->DB->get_updated_rows( $source[0]['projectid'], $total_posts, $idcolumn);
        
        if( !$updated_rows ){
            $this->UI->create_notice( __( 'None of your imported rows have been 
            updated since their original import. This notice will not
            be displayed to anyone during automated post updating and is for 
            test purposes only.' ), 'info', 'Small', 
            __( 'Test Complete: No Posts Updated', '' ) );
            return;
        }
            
        $foreach_done = 0;
        foreach( $updated_rows as $key => $row){
            ++$foreach_done;
                        
            // to get the output at the end, tell the class we are on the final post, only required in "manual" requestmethod
            if( $foreach_done == $total_posts ){
                $autoblog->finished = true;
            }            
            // pass row to $autob
            $autoblog->row = $row;    
            // create a post - start method is the beginning of many nested functions
            $autoblog->start();
        } 
        
        // log
        $atts = array(
            'outcome' => 1,
            'line' => __LINE__, 
            'function' => __FUNCTION__,
            'file' => __FILE__, 
            'userid' => get_current_user_id(),          
            'type' => 'general',
            'category' => 'postsevent',
            'action' => __( "Up to $foreach_done posts may have been updated.", 'csv2post' ),
            'priority' => 'normal',                        
            'triga' => 'manualrequest'
        );

        $this->LOG->newlog( $atts );          
            $this->UI->create_notice( __( 'Posts may have been updated. This 
            notice will not be generated for any user during automated events and
            is only to help make testing clearer.' ), 'info', 'Small', 
            __( 'Testing Complete: Posts Updated', 'csv2post' ) );            
            return false;
    }
        
}
?>
